@@ -22,7 +22,11 @@ class TreeTests(TestCase):
         deep_node = tree.loc['a']
         self.assertEqual(deep_node.root, tree)
 
-    # def test_quoted_fasttree_newick(self):
-    #    Tree.from_newick(
-    #        filename='./tests/data/fasttree.quoted_labels.newick'
-    #    )
+    '''
+    def test_quoted_fasttree_newick(self):
+        print(__file__)
+        print('hi')
+        Tree.from_newick(
+            filename='./tests/data/fasttree.quoted_labels.newick'
+        )
+    '''
